@@ -1,10 +1,11 @@
 package OpenGuides::Test;
 
 use OpenGuides::Config;
+use Wiki::Toolkit::Setup::SQLite;
 
 use strict;
 use vars qw( $VERSION );
-$VERSION = '0.02';
+$VERSION = '0.06';
 
 use CGI;
 
@@ -24,6 +25,8 @@ only useful to OpenGuides developers.
   use OpenGuides;
   use OpenGuides::Test;
 
+  OpenGuides::Test::refresh_db();
+
   my $config = OpenGuides::Test->make_basic_config;
   $config->default_language( "nl" );
 
@@ -65,6 +68,8 @@ sub make_basic_config {
                      home_name            => "Home",
                      geo_handler          => 1,
                      force_wgs84          => 1,
+                     contact_email        => 'admins@example.org',
+                     moderate_whitelist   => "",
                    }
     );
 
@@ -159,18 +164,18 @@ sub write_data {
     }
 }
 
-=over 4
-
 =item B<make_cgi_object>
 
   my $q = OpenGuides::Test->make_cgi_object;
 
-You can supply values for the following keys: C<content>,
-C<categories>, C<locales>, C<os_x>, C<os_y>, C<osie_x>, C<osie_y>,
-C<latitude>, C<longitude>, C<summary>, C<node_image>, C<node_image_licence>,
-C<node_image_copyright>, C<node_image_url>, C<username>, C<comment>,
-C<edit_type>.  You should supply them exactly as they would come from a CGI
-form, eg lines in a textarea are separated by C<\r\n>.
+You can supply values for the following keys: C<content>, C<categories>,
+C<locales>, C<node_image>, C<node_image_licence>,
+C<node_image_copyright>, C<node_image_url>, C<phone>, C<fax>,
+C<website>, C<hours_text>, C<address>, C<postcode>, C<map_link>,
+C<os_x>, C<os_y>, C<osie_x>, C<osie_y>, C<latitude>, C<longitude>,
+C<summary>, C<username>, C<comment>, C<edit_type>.  You should supply
+them exactly as they would come from a CGI form, eg lines in a textarea
+are separated by C<\r\n>.
 
 =cut
 
@@ -180,39 +185,39 @@ sub make_cgi_object {
     # Set up CGI parameters ready for a node write.
     # Most of these are in here to avoid uninitialised value warnings.
     my $q = CGI->new( "" );
-    $q->param( -name => "content", -value => $args{content} || "foo" );
-    $q->param( -name => "categories", -value => $args{categories} || "" );
-    $q->param( -name => "locales", -value => $args{locales} || "" );
-    $q->param( -name => "node_image", -value => $args{node_image} || "" );
-    $q->param( -name => "node_image_licence",
-               -value => $args{node_image_licence} || "" );
-    $q->param( -name => "node_image_copyright",
-               -value => $args{node_image_copyright} || "" );
-    $q->param( -name => "node_image_url",
-               -value => $args{node_image_url} || "" );
-    $q->param( -name => "phone", -value => "" );
-    $q->param( -name => "fax", -value => "" );
-    $q->param( -name => "website", -value => "" );
-    $q->param( -name => "hours_text", -value => "" );
-    $q->param( -name => "address", -value => "" );
-    $q->param( -name => "postcode", -value => "" );
-    $q->param( -name => "map_link", -value => "" );
-    $q->param( -name => "os_x", -value => $args{os_x} || "" );
-    $q->param( -name => "os_y", -value => $args{os_y} || "" );
-    $q->param( -name => "osie_x", -value => $args{osie_x} || "" );
-    $q->param( -name => "osie_y", -value => $args{osie_y} || "" );
-    $q->param( -name => "latitude", -value => $args{latitude} || "" );
-    $q->param( -name => "longitude", -value => $args{longitude} || "" );
-    $q->param( -name => "summary", -value => $args{summary} || "" );
-    $q->param( -name => "username", -value => $args{username} || "TestUser" );
-    $q->param( -name => "comment", -value => $args{comment} || "A comment." );
-    $q->param( -name => "edit_type",
-               -value => $args{edit_type} || "Normal edit" );
+    $args{content} ||= "foo";
+    $args{edit_type} ||= "Normal edit";
+    for my $param ( qw( content categories locales node_image node_image_licence
+                        node_image_copyright node_image_url phone fax website
+                        hours_text address postcode map_link os_x os_y osie_x osie_y
+                        latitude longitude summary username comment edit_type
+                      )
+                  ) {
+        if (defined $args{$param}) {
+            $q->param( -name => $param, -value => $args{$param} );
+        } else {
+            $q->param( -name => $param, -value => '' );
+        }
+    }
     $ENV{REMOTE_ADDR} = "127.0.0.1";
 
     return $q;
 }
 
+=item B<refresh_db>
+
+  Openguides::Test::refresh_db();
+
+Unlink the existing SQLite database t/node.db and plucene indexes. Then create a new SQLite database t/node.db 
+
+=cut 
+sub refresh_db {
+    unlink "t/node.db";
+    unlink <t/indexes/*>;
+    Wiki::Toolkit::Setup::SQLite::setup( { dbname => "t/node.db" } );
+}
+
+
 =back
 
 =head1 AUTHOR
@@ -221,7 +226,7 @@ The OpenGuides Project (openguides-dev@lists.openguides.org)
 
 =head1 COPYRIGHT
 
-  Copyright (C) 2004-2007 The OpenGuides Project.  All Rights Reserved.
+  Copyright (C) 2004-2009 The OpenGuides Project.  All Rights Reserved.
 
 This module is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
